
/* GENERATED CODE... DO NOT EDIT */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * Copyright (C) 2013 - 2014 Aleksander Morgado <aleksander@aleksander.es>
 */


#include <string.h>

#include "mbim-ussd.h"
#include "mbim-message-private.h"
#include "mbim-enum-types.h"
#include "mbim-error-types.h"
#include "mbim-device.h"
#include "mbim-utils.h"

/*****************************************************************************/
/* Message (Set): MBIM Message USSD */

/**
 * mbim_message_ussd_set_new:
 * @action: the 'Action' field, given as a #MbimUssdAction.
 * @data_coding_scheme: the 'DataCodingScheme' field, given as a #guint32.
 * @payload_size: size of the payload array.
 * @payload: the 'Payload' field, given as an array of #guint8 values.
 * @error: return location for error or %NULL.
 *
 * Create a new request for the '' set command in the 'USSD' service.
 *
 * Returns: a newly allocated #MbimMessage, which should be freed with mbim_message_unref().
 */
MbimMessage *
mbim_message_ussd_set_new (
    MbimUssdAction action,
    guint32 data_coding_scheme,
    const guint32 payload_size,
    const guint8 *payload,
    GError **error)
{
    MbimMessageCommandBuilder *builder;

    builder = _mbim_message_command_builder_new (0,
                                                 MBIM_SERVICE_USSD,
                                                 MBIM_CID_USSD,
                                                 MBIM_MESSAGE_COMMAND_TYPE_SET);
    {
        _mbim_message_command_builder_append_guint32 (builder, action);
    }
    {
        _mbim_message_command_builder_append_guint32 (builder, data_coding_scheme);
    }
    {
        _mbim_message_command_builder_append_byte_array (builder, TRUE, TRUE, payload, payload_size);
    }

    return _mbim_message_command_builder_complete (builder);
}

/*****************************************************************************/
/* Message (Response): MBIM Message USSD */

/**
 * mbim_message_ussd_response_parse:
 * @message: the #MbimMessage.
 * @response: return location for a #MbimUssdResponse, or %NULL if the 'Response' field is not needed.
 * @session_state: return location for a #MbimUssdSessionState, or %NULL if the 'SessionState' field is not needed.
 * @data_coding_scheme: return location for a #guint32, or %NULL if the 'DataCodingScheme' field is not needed.
 * @payload_size: return location for the size of the payload array.
 * @payload: return location for an array of #guint8 values. Do not free the returned value, it is owned by @message.
 * @error: return location for error or %NULL.
 *
 * Create a new request for the 'Payload' response command in the 'USSD' service.
 *
 * Returns: %TRUE if the message was correctly parsed, %FALSE if @error is set.
 */
gboolean
mbim_message_ussd_response_parse (
    const MbimMessage *message,
    MbimUssdResponse *response,
    MbimUssdSessionState *session_state,
    guint32 *data_coding_scheme,
    guint32 *payload_size,
    const guint8 **payload,
    GError **error)
{
    guint32 offset = 0;

    if (mbim_message_get_message_type (message) != MBIM_MESSAGE_TYPE_COMMAND_DONE) {
        g_set_error (error,
                     MBIM_CORE_ERROR,
                     MBIM_CORE_ERROR_INVALID_MESSAGE,
                     "Message is not a response");
        return FALSE;
    }

    /* Read the 'Response' variable */
    {
        if (response != NULL)
            *response =  _mbim_message_read_guint32 (message, offset);
        offset += 4;
    }

    /* Read the 'SessionState' variable */
    {
        if (session_state != NULL)
            *session_state =  _mbim_message_read_guint32 (message, offset);
        offset += 4;
    }

    /* Read the 'DataCodingScheme' variable */
    {
        if (data_coding_scheme != NULL)
            *data_coding_scheme =  _mbim_message_read_guint32 (message, offset);
        offset += 4;
    }

    /* Read the 'Payload' variable */
    {
        const guint8 *tmp;
        guint32 tmpsize;

        tmp = _mbim_message_read_byte_array (message, 0, offset, TRUE, TRUE, &tmpsize);
        if (payload != NULL)
            *payload = tmp;
        if (payload_size != NULL)
            *payload_size = tmpsize;
        offset += 8;
    }

    return TRUE;
}

/*****************************************************************************/
/* Message (Notification): MBIM Message USSD */

/**
 * mbim_message_ussd_notification_parse:
 * @message: the #MbimMessage.
 * @response: return location for a #MbimUssdResponse, or %NULL if the 'Response' field is not needed.
 * @session_state: return location for a #MbimUssdSessionState, or %NULL if the 'SessionState' field is not needed.
 * @data_coding_scheme: return location for a #guint32, or %NULL if the 'DataCodingScheme' field is not needed.
 * @payload_size: return location for the size of the payload array.
 * @payload: return location for an array of #guint8 values. Do not free the returned value, it is owned by @message.
 * @error: return location for error or %NULL.
 *
 * Create a new request for the 'Payload' notification command in the 'USSD' service.
 *
 * Returns: %TRUE if the message was correctly parsed, %FALSE if @error is set.
 */
gboolean
mbim_message_ussd_notification_parse (
    const MbimMessage *message,
    MbimUssdResponse *response,
    MbimUssdSessionState *session_state,
    guint32 *data_coding_scheme,
    guint32 *payload_size,
    const guint8 **payload,
    GError **error)
{
    guint32 offset = 0;

    if (mbim_message_get_message_type (message) != MBIM_MESSAGE_TYPE_INDICATE_STATUS) {
        g_set_error (error,
                     MBIM_CORE_ERROR,
                     MBIM_CORE_ERROR_INVALID_MESSAGE,
                     "Message is not a notification");
        return FALSE;
    }

    /* Read the 'Response' variable */
    {
        if (response != NULL)
            *response =  _mbim_message_read_guint32 (message, offset);
        offset += 4;
    }

    /* Read the 'SessionState' variable */
    {
        if (session_state != NULL)
            *session_state =  _mbim_message_read_guint32 (message, offset);
        offset += 4;
    }

    /* Read the 'DataCodingScheme' variable */
    {
        if (data_coding_scheme != NULL)
            *data_coding_scheme =  _mbim_message_read_guint32 (message, offset);
        offset += 4;
    }

    /* Read the 'Payload' variable */
    {
        const guint8 *tmp;
        guint32 tmpsize;

        tmp = _mbim_message_read_byte_array (message, 0, offset, TRUE, TRUE, &tmpsize);
        if (payload != NULL)
            *payload = tmp;
        if (payload_size != NULL)
            *payload_size = tmpsize;
        offset += 8;
    }

    return TRUE;
}
